/* --- General element styles --- */

html {
    min-height: 100%;  /* Allow background to fill entire screen */
}


/* --- Bootstrap overrides --- */

.form-control:focus {
    box-shadow: none;
    border-color: darkgray;
}


/* --- Camus specific --- */

#control-bar {
    margin-top: 10px;
    margin-bottom: 10px;
    text-align: center;
}

#video-stage {
    border: 1px dotted lightgray;
    border-radius: 0.5em;
    min-height: 480px;
}

#video-thumbs {
    max-height: 600;
    overflow-y: auto;
    text-overflow: ellipsis;
}

#message-bar {
    width: 100%;
    height: 100%;
}

#message-log {
    min-height: 200px;
    max-height: 400px;
    overflow-y: auto;
}

.content {
    margin-top: 20px;
}

.boldlink {
    color: darkslategray;
}

.boldlink:hover,
.boldlink:focus {
    color: darkslategray;
    text-shadow: 0.1px 0.1px 0px gray,
                 -0.1px -0.1px 0px gray,
                 0.1px -0.1px 0px gray,
                 -0.1px 0.1px 0px gray;
    text-decoration: none;
}

.icon-button:hover,
.icon-button:focus {
    background-color: #eeeeee;
    outline: none;
    box-shadow: none;
}

.icon-button:active {
    background-color: #dddddd;
}

.video-box {
    position: relative;
    box-shadow: inset 0 0 4em 2em #000000;
    border-radius: 0.5em;
}

.video-box video {
    /* Fill the video-box */
    height: 100%;
    width: 100%;
    display: flex;

    max-height: 750px;
    background-color: #000;
    border-radius: 0.5em;
}

.video-box:hover video {
    opacity: 0.7;
    transition: opacity 0.2s;
    transition-delay: 0.2s;
    transition-timing-function: ease;
}

.video-box .video-tag {
    display: none;
    position: absolute;
    left: 20px;
    top: 10px;
    z-index: 9;
    max-width: 80%;
    text-overflow: ellipsis;
    font-size: 1.5em;
}

.video-box:hover .video-tag {
    display: inline;
    color: #000;
    text-shadow: 1px 1px 4px #fff,
                 -1px -1px 4px #fff;
                 1px -1px 4px #fff;
                 -1px 1px 4px #fff;
    text-overflow: ellipsis;
}

.message-time {
    display: inline-block;
    font-size: small;
    font-style: italic;
    margin-bottom: 0px;
    margin-left: 10px;
    float: right;
}

.message-from {
    display: inline-block;
    font-variant-caps: all-small-caps;
    font-weight: bold;
    margin-bottom: 0px;
}

.message-text {
    margin-top: 0px;
}

.tech-info {
    font-variant-caps: all-small-caps;
    font-weight: bold;
    margin-bottom: 0px;
}

.tech-info > span {
    font-variant-caps: normal;
    font-weight: normal;
    font-size: small;
    font-style: italic;
}

.about a {
    color: #004B6B;
    text-decoration: none;
    border-bottom: 1px dotted #999;
}

.about a:hover {
    color: #6B6000;
    text-decoration: none;
    border-bottom: 1px solid #999;
}

.about i {
    /* Consistent sizing for material icons */
    font-size: 1em;
}
