<!DOCTYPE html>
<html{% block html_attribs %}{% endblock html_attribs %}>
{%- block html %}
  <head>
    {%- block head %}
    <title>{% block title %}{{title|default}}{% endblock title %}</title>

    {%- block metas %}
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0, shrink-to-fit=no">
    <meta name="description" content="Peer-to-peer video chat application using WebRTC. Audio and video streaming, desktop sharing, and text chat.">
    {% block robots %}<meta name="robots" content="index, follow">{% endblock robots %}
    {%- endblock metas %}

    {%- block styles %}
        <!-- Bootstrap CSS -->
        {{ bootstrap.load_css() }}
    {%- endblock styles %}
    {%- endblock head %}
  </head>
  <body{% block body_attribs %}{% endblock body_attribs %}>
    {% block body -%}
    {% block navbar %}
    {%- endblock navbar %}
    {% block content -%}
    {%- endblock content %}

    {% block scripts %}
    {{ bootstrap.load_js() }}
    {%- endblock scripts %}
    {%- endblock body %}
  </body>
{%- endblock html %}
</html>
